@@ -2,14 +2,14 @@ import os
 
 import sh
 
-from fedeproxy.interfaces.dvcs import DVCS
+from .dvcs import DVCS
 
 
 class Hg(DVCS):
     def __init__(self, dir, url):
         self._url = url
         self.d = dir
-        self.h = sh.hg.bake("--cwd", self.d)
+        self.h = sh.hg.bake("--cwd", self.d, _tty_out=False)
 
     @property
     def directory(self):
@@ -19,6 +19,29 @@ class Hg(DVCS):
     def url(self):
         return self._url
 
+    def set_url_from_directory(self):
+        self._url = self.h.paths.default().strip()
+
+    @property
+    def hash(self):
+        return self.h.identify("-i").strip()
+
+    def fetch(self, url, hash):
+        try:
+            self.h.identify("-r", hash)
+            return False
+        except sh.ErrorReturnCode_255 as e:
+            if "unknown revision" not in e.args[0]:
+                raise
+        self.h.pull("-f", url)
+        self.h.identify("-r", hash)
+        return True
+
+    def reset(self, branch, hash):
+        self.clone(branch)
+        self.h.update("--clean", "-r", hash)
+        return True
+
     def clone(self, branch):
         clone = not os.path.exists(self.d)
         if clone:
