@@ -6,6 +6,7 @@ import requests
 
 from fedeproxy.common.retry import retry
 
+from ..dvcs.git import Git
 from . import forge
 
 logger = logging.getLogger(__name__)
@@ -46,77 +47,18 @@ class Gitea(forge.Forge):
         r.raise_for_status()
         self.set_token(r.json()["sha1"])
 
-    def _finalize_user_create(self, username, password):
-        r = self.s.post(
-            f"{self.url}/user/login",
-            data={
-                "user_name": username,
-                "password": password,
-            },
-        )
-        r.raise_for_status()
-        r = self.s.post(
-            f"{self.url}/user/settings/change_password",
-            data={
-                "password": password,
-                "retype": password,
-                "_csrf": self.s.cookies["_csrf"],
-            },
-        )
-        r.raise_for_status()
-
     def get_token(self):
         return self.token
 
+    def is_dvcs_supported(self, DVCS):
+        return DVCS == Git
+
     def set_token(self, token):
         self.token = token
         self.s.headers["Authorization"] = f"token {token}"
 
-    def user_delete(self, username):
-        user = self.user_get(username)
-        if user is None:
-            return False
-        while True:
-            r = self.s.delete(f"{self.s.api}/admin/users/{username}")
-            if r.status_code == 404:
-                break
-            logger.debug(r.text)
-            r.raise_for_status()
-        return True
-
-    def user_factory(self):
-        return GiteaUser
-
-    def user_get(self, username):
-        if username == self.username:
-            r = self.s.get(f"{self.s.api}/user")
-        elif self.is_admin:
-            r = self.s.get(f"{self.s.api}/user", params={"sudo": username})
-        else:
-            r = self.s.get(f"{self.s.api}/users/{username}")
-        if r.status_code == 404:
-            return None
-        else:
-            r.raise_for_status()
-            return self.user_factory()(self, r.json())
-
-    def user_create(self, username, password, email):
-        info = self.user_get(username)
-        if info is None:
-            r = self.s.post(
-                f"{self.s.api}/admin/users",
-                data={
-                    "username": username,
-                    "email": email,
-                    "password": password,
-                },
-            )
-            logger.debug(r.text)
-            r.raise_for_status()
-            info = r.json()
-            Gitea(self.url)._finalize_user_create(username, password)
-            info = self.user_get(username)
-        return info
+    def users_factory(self):
+        return GiteaUsers
 
     def project_delete(self, namespace, project):
         p = self.project_get(namespace, project)
@@ -243,7 +185,10 @@ class GiteaMilestones(forge.Milestones):
         r.raise_for_status()
 
     def list(self):
-        pass
+        r = self.s.get(self._build_url())
+        r.raise_for_status()
+        for m in r.json():
+            yield GiteaMilestone(self.project, m)
 
 
 class GiteaMilestone(forge.Milestone):
@@ -278,6 +223,78 @@ class GiteaIssue(forge.Issue):
         pass
 
 
+class GiteaUsers(forge.Users):
+    def get(self, username):
+        if username == self.forge.username:
+            r = self.s.get(f"{self.s.api}/user")
+        elif self.forge.is_admin:
+            r = self.s.get(f"{self.s.api}/user", params={"sudo": username})
+        else:
+            r = self.s.get(f"{self.s.api}/users/{username}")
+        if r.status_code == 404:
+            return None
+        else:
+            r.raise_for_status()
+            return GiteaUser(self.forge, r.json())
+
+    def delete(self, username):
+        user = self.get(username)
+        if user is None:
+            return False
+        while True:
+            r = self.s.delete(f"{self.s.api}/admin/users/{username}")
+            if r.status_code == 404:
+                break
+            logger.debug(r.text)
+            r.raise_for_status()
+        return True
+
+    def create(self, username, password, email):
+        info = self.get(username)
+        if info is None:
+            r = self.s.post(
+                f"{self.s.api}/admin/users",
+                data={
+                    "username": username,
+                    "email": email,
+                    "password": password,
+                },
+            )
+            logger.debug(r.text)
+            r.raise_for_status()
+            info = r.json()
+            Gitea(self.forge.url).users._finalize_user_create(username, password)
+            info = self.get(username)
+        return info
+
+    def _finalize_user_create(self, username, password):
+        r = self.s.post(
+            f"{self.forge.url}/user/login",
+            data={
+                "user_name": username,
+                "password": password,
+            },
+        )
+        r.raise_for_status()
+        r = self.s.post(
+            f"{self.forge.url}/user/settings/change_password",
+            data={
+                "password": password,
+                "retype": password,
+                "_csrf": self.s.cookies["_csrf"],
+            },
+        )
+        r.raise_for_status()
+
+    def list(self):
+        r = self.s.get(f"{self.s.api}/users/search")
+        r.raise_for_status()
+        j = r.json()
+        assert j["ok"]
+        for u in j["data"]:
+            yield GiteaUser(self.forge, u)
+
+
 class GiteaUser(forge.User):
     @property
     def url(self):
@@ -290,3 +307,12 @@ class GiteaUser(forge.User):
     @property
     def emails(self):
         return [self._user["email"]]
+
+    def to_json(self):
+        return {
+            "url": self.url,
+            "username": self.username,
+        }
+
+    def from_json(self, j):
+        pass
