@@ -7,6 +7,8 @@ import requests
 
 from fedeproxy.common.retry import retry
 
+from ..dvcs.git import Git
+from ..dvcs.hg import Hg
 from . import forge
 
 logger = logging.getLogger(__name__)
@@ -48,6 +50,9 @@ class GitLab(forge.Forge):
     def get_token(self):
         return self.token
 
+    def is_dvcs_supported(self, DVCS):
+        return DVCS in (Git, Hg)
+
     def set_token(self, token):
         self.token = token
 
@@ -55,49 +60,8 @@ class GitLab(forge.Forge):
         r = self.s.namespaces.list(search=name)
         return r[0].id
 
-    def user_delete(self, username):
-        user = self.user_get(username)
-        if user is None:
-            return False
-        while True:
-            try:
-                self.s.users.delete(user.id)
-            except gitlab.exceptions.GitlabDeleteError as e:
-                if e.response_code == 404:
-                    break
-                raise
-            time.sleep(0.1)
-        return True
-
-    def user_factory(self):
-        return GitLabUser
-
-    def user_get(self, username):
-        if username == self.username:
-            return self.user_factory()(self, self.s.user)
-
-        for found in self.s.users.list(username=username):
-            #
-            # There may be more than one match because the search is case insensitive
-            #
-            if found.username == username:
-                return self.user_factory()(self, found)
-        return None
-
-    def user_create(self, username, password, email):
-        info = self.user_get(username)
-        if info is None:
-            self.s.users.create(
-                {
-                    "name": username,
-                    "username": username,
-                    "email": email,
-                    "password": password,
-                    "skip_confirmation": True,
-                }
-            )
-            info = self.user_get(username)
-        return info
+    def users_factory(self):
+        return GitLabUsers
 
     def project_delete(self, namespace, project):
         p = self.project_get(namespace, project)
@@ -159,7 +123,7 @@ class GitLabProject(forge.Project):
 
     @property
     def namespace(self):
-        return self._project.namespace.path
+        return self._project.namespace["path"]
 
     @property
     def project(self):
@@ -182,22 +146,36 @@ class GitLabProject(forge.Project):
 
 class GitLabMilestones(forge.Milestones):
     def get(self, id):
-        pass
+        try:
+            return self._project._project.milestones.get(id)
+        except gitlab.exceptions.GitlabGetError as e:
+            if e.response_code == 404:
+                return None
+            else:
+                raise
 
     def delete(self, id):
-        pass
+        try:
+            self._project._project.milestones.delete(id)
+        except gitlab.exceptions.GitlabDeleteError as e:
+            if e.response_code == 404:
+                return False
+            else:
+                raise
+        return True
 
     def create(self, title, **data):
-        pass
+        return self._project._project.milestones.create({"title": title})
 
     def list(self):
-        pass
+        for milestone in self._project._project.milestones.list():
+            yield GitLabMilestone(self.project, milestone)
 
 
 class GitLabMilestone(forge.Milestone):
     @property
     def id(self):
-        pass
+        return self._milestone.id
 
 
 class GitLabIssues(forge.Issues):
@@ -257,6 +235,53 @@ class GitLabIssue(forge.Issue):
         self._issue.save()
 
 
+class GitLabUsers(forge.Users):
+    def get(self, username):
+        if username == self.forge.username:
+            return GitLabUser(self, self.s.user)
+
+        for found in self.s.users.list(username=username):
+            #
+            # There may be more than one match because the search is case insensitive
+            #
+            if found.username == username:
+                return GitLabUser(self.forge, found)
+        return None
+
+    def delete(self, username):
+        user = self.get(username)
+        if user is None:
+            return False
+        while True:
+            try:
+                self.s.users.delete(user.id)
+            except gitlab.exceptions.GitlabDeleteError as e:
+                if e.response_code == 404:
+                    break
+                raise
+            time.sleep(0.1)
+        return True
+
+    def create(self, username, password, email):
+        info = self.get(username)
+        if info is None:
+            self.s.users.create(
+                {
+                    "name": username,
+                    "username": username,
+                    "email": email,
+                    "password": password,
+                    "skip_confirmation": True,
+                }
+            )
+            info = self.get(username)
+        return info
+
+    def list(self):
+        for u in self.s.users.list():
+            yield GitLabUser(self.forge, u)
+
+
 class GitLabUser(forge.User):
     @property
     def url(self):
@@ -278,3 +303,12 @@ class GitLabUser(forge.User):
     @property
     def id(self):
         return self._user.id
+
+    def to_json(self):
+        return {
+            "url": self.url,
+            "username": self.username,
+        }
+
+    def from_json(self, j):
+        pass
