@@ -11,12 +11,12 @@ def test_gitea_fork():
 
     username1 = "testuser1"
     email1 = "testuser1@example.com"
-    forge.user_create(username1, password, email1)
+    forge.users.create(username1, password, email1)
     forge.project_delete(username1, "testproject")
 
     username2 = "testuser2"
     email2 = "testuser2@example.com"
-    forge.user_create(username2, password, email2)
+    forge.users.create(username2, password, email2)
     forge.project_delete(username2, "testproject")
 
     forge.authenticate(username=username1, password=password)
@@ -33,6 +33,6 @@ def test_gitea_fork():
 
     forge.authenticate(username="root", password="Wrobyak4")
     assert forge.project_delete(username2, "testproject") is True
-    assert forge.user_delete(username2) is True
+    assert forge.users.delete(username2) is True
     assert forge.project_delete(username1, "testproject") is True
-    assert forge.user_delete(username1) is True
+    assert forge.users.delete(username1) is True
