@@ -16,6 +16,23 @@ class DVCS(abc.ABC):
     def url(self):
         ...
 
+    @abc.abstractmethod
+    def set_url_from_directory(self):
+        ...
+
+    @property
+    @abc.abstractmethod
+    def hash(self):
+        ...
+
+    @abc.abstractmethod
+    def fetch(self, url, hash):
+        ...
+
+    @abc.abstractmethod
+    def reset(self, hash):
+        ...
+
     @abc.abstractmethod
     def clone(self, branch):
         ...
