@@ -16,18 +16,24 @@ Including another URLconf
 from django.conf import settings
 from django.conf.urls import include, url
 from django.conf.urls.static import static
-from django.urls import path
+from django.urls import re_path
 from django.views.generic import TemplateView
 
-from fedeproxy.api import views
+from fedeproxy.activitypub import regex
+from fedeproxy.views import activitypub, wellknown
 
-from .views import member_index
+USER_PATH = r"^user/(?P<username>%s)" % regex.USERNAME
+LOCAL_USER_PATH = r"^user/(?P<username>%s)" % regex.LOCALNAME
 
 # Wire up our API using automatic URL routing.
 # Additionally, we include login URLs for the browsable API.
 urlpatterns = [
-    path("ping/", views.ping, name="ping"),
     url(r"^$", TemplateView.as_view(template_name="visitor/landing-index.html"), name="landing_index"),
     url(r"^accounts/", include("allauth.urls")),
-    url(r"^member/$", member_index, name="user_home"),
+    re_path(r"^\.well-known/nodeinfo/?$", wellknown.nodeinfo_pointer, name="wellknown-nodeinfo"),
+    re_path(r"^nodeinfo/2\.0/?$", wellknown.nodeinfo),
+    re_path(r"^inbox/?$", activitypub.inbox),
+    re_path(r"%s/inbox/?$" % LOCAL_USER_PATH, activitypub.inbox, name="inbox"),
+    re_path(r"%s/outbox/?$" % LOCAL_USER_PATH, activitypub.outbox),
+    re_path(r"%s/?$" % USER_PATH, activitypub.user, name="user"),
 ] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)
